@@ -1,13 +1,12 @@
 /*
  *  resourcedatamodelbase.h  -  base for models containing calendars and events
  *  Program:  kalarm
- *  SPDX-FileCopyrightText: 2007-2020 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2007-2021 David Jarvie <djarvie@kde.org>
  *
  *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#ifndef RESOURCEDATAMODELBASE_H
-#define RESOURCEDATAMODELBASE_H
+#pragma once
 
 #include "resourcetype.h"
 
@@ -87,6 +86,15 @@ public:
     /** Return offset to add to headerData() role, for item models. */
     virtual int headerDataEventRoleOffset() const  { return 0; }
 
+    /** Return the alarm time text in the form "date time".
+     *  @param dateTime     the date/time to format.
+     *  @param leadingZero  the character to represent a leading zero, or '\0' for no leading zeroes.
+     */
+    static QString alarmTimeText(const DateTime& dateTime, char leadingZero = '\0');
+
+    /** Return the time-to-alarm text. */
+    static QString timeToAlarmText(const DateTime&);
+
 protected:
     ResourceDataModelBase();
 
@@ -184,6 +192,5 @@ private:
 friend class DataModel;
 };
 
-#endif // RESOURCEDATAMODELBASE_H
 
 // vim: et sw=4:
